using System;
using System.Net.Sockets;
using System.Timers;
using System.IO;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace LabClient_cs
{
    class Program
    {
        static int atoi(string s) { return int.Parse(s, CultureInfo.InvariantCulture); }

        static void event_loop(TcpClient cli, StreamReader reader, StreamWriter writer)
        {
            try
            {
                char[] delimiter = new char[] { ' ' };

                while (true)
                {
                    string cmd = reader.ReadLine();
                    string[] cmda = cmd.Split(delimiter);

                    if (cmda[0] == "ADIJO") return;
                    if (cmda[0].StartsWith("KONEC_IGRE")) continue;
                    if (cmda[0] != "ZBUDI_SE") throw new Exception("Napaka pri komunikaciji s streznikom.");

                    int visina = atoi(cmda[2]), sirina = atoi(cmda[1]), vidljivost = atoi(cmda[3]);
                    Robot t = new Robot();
                    t.ZbudiSe(sirina, visina, vidljivost);

                    writer.WriteLine("BUDEN");

                    DateTime zacetek = DateTime.Now;

                    while (true)
                    {
                        cmd = reader.ReadLine();
                        cmda = cmd.Split(delimiter);

                        if (cmda[0] == "ADIJO") return;
                        if (cmda[0] == "KONEC_IGRE")
                        {
                            ZakajKonec razlog;
                            if (cmda[1] == "ZMAGA") razlog = ZakajKonec.Zmaga;
                            else if (cmda[1] == "NAPAKA") razlog = ZakajKonec.Napaka;
                            else razlog = ZakajKonec.Drugo;
                            string opis = ""; for (int i = 2; i < cmda.Length; i++) opis += (i == 2 ? "" : " ") + cmda[i];
                            t.KonecIgre(razlog, opis);
                            t = null;
                            break;
                        }
                        else if (cmda[0] == "QV")
                        {
                            bool nosimBreme = (cmda[1] == "B");
                            int nVidenih = atoi(cmda[2]);
                            VidenoPolje[] kajVidim = new VidenoPolje[nVidenih];
                            for (int i = 0; i < nVidenih; i++)
                            {
                                VidenoPolje vp;
                                vp.rowsAhead = atoi(cmda[3 * i + 3]);
                                vp.column = atoi(cmda[3 * i + 4]);
                                string s = cmda[3 * i + 5];
                                if (s == ".") vp.tip = TipPolja.Prosto;
                                else if (s == "x") vp.tip = TipPolja.Izhod;
                                else if (s == "Z") vp.tip = TipPolja.ZunanjiZid;
                                else if (s == "#") vp.tip = TipPolja.KockaAliZid;
                                else throw new Exception("Napaka pri komunikaciji s streznikom: neznan tip polja \"" + s + "\".");
                                kajVidim[i] = vp;
                            }

                            Timer timeout = new Timer(1000000 - (DateTime.Now - zacetek).Milliseconds);
                            timeout.Elapsed += new ElapsedEventHandler(timeout_Elapsed);
                            timeout.Start();

                            Premik premik = t.KamGres(nosimBreme, kajVidim);

                            timeout.Stop();

                            string p;
                            if (premik == Premik.Naprej) p = "NAPREJ";
                            else if (premik == Premik.Levo) p = "LEVO";
                            else if (premik == Premik.Desno) p = "DESNO";
                            else if (premik == Premik.Poberi) p = "POBERI";
                            else if (premik == Premik.Spusti) p = "SPUSTI";
                            else if (premik == Premik.Obrni) p = "UTURN";
                            else throw new Exception("Napaka v robotu: neznan premik " + premik.ToString());
                            writer.WriteLine("QVR " + p);
                        }
                        else throw new Exception("Napaka pri komunikaciji s streznikom: neznan tip sporocila \"" + cmda[0] + "\".");
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Napaka: " + e.ToString());
            }
        }

        static void timeout_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            System.Environment.Exit(0);
        }

        static TcpClient connect(string hostname, int portNo, out StreamReader reader, out StreamWriter writer)
        {
            TcpClient client = new TcpClient(hostname, portNo);
            reader = new StreamReader(client.GetStream());
            writer = new StreamWriter(client.GetStream());
            writer.AutoFlush = true;

            string server = reader.ReadLine();
            Console.WriteLine(server);

            writer.WriteLine("AVE " + Robot.Ime);
            string succ = reader.ReadLine();
            if (succ != "PRIJAVA USPESNA") throw new Exception("Prijava na streznik neuspesna: " + succ);
            return client;
        }

        static void Main(string[] args)
        {
            try
            {
                string hostname; int portNo;
                if (args.Length < 1) hostname = "localhost";
                else hostname = args[0];
                if (args.Length < 2) portNo = 1176;
                else portNo = int.Parse(args[1]);

                StreamReader reader;
                StreamWriter writer;
                TcpClient client = connect(hostname, portNo, out reader, out writer);
                event_loop(client, reader, writer);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }
    }
}
